"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const webpack_1 = tslib_1.__importDefault(require("webpack"));
const path_1 = tslib_1.__importDefault(require("path"));
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const child_process_1 = require("child_process");
const utils_1 = require("./utils");
const webpack_config_1 = require("../config/webpack-config");
const config_1 = require("../config/config");
process.env.NODE_ENV = 'development';
const config = config_1.getMergedConfig();
let childProcess = null;
let chunkPath = '';
let buildTime = 0;
let hasError = false;
function createChildProcess() {
    childProcess = child_process_1.fork(chunkPath, [], { stdio: 'inherit' });
    childProcess.on('error', (err) => {
        console.error(err.message);
        process.exit(1);
    });
    childProcess.on('close', () => {
        var _a;
        config.cleanConsoleOnRebuild && console.clear();
        if (((_a = childProcess) === null || _a === void 0 ? void 0 : _a.killed) && !hasError) {
            createChildProcess();
        }
    });
    console.log(chalk_1.default.green(`Compiled successfully! Done in ${(buildTime / 1000).toFixed(2)}s`));
}
function runCompile(configs) {
    const compiler = webpack_1.default(configs);
    compiler.hooks.watchRun.tap('ssr-watch-run', () => {
        config.cleanConsoleOnRebuild && console.clear();
        console.log(chalk_1.default.green(`Compiling...`));
    });
    compiler.watch({ ignored: /node_modules/ }, (err, stats) => {
        var _a, _b, _c;
        if (err) {
            hasError = true;
            (_a = childProcess) === null || _a === void 0 ? void 0 : _a.kill(1);
            throw err;
        }
        const [serverStats] = stats.stats;
        const { errors, warnings, outputPath, assetsByChunkName, time } = serverStats.toJson();
        if (warnings.length) {
            console.log(chalk_1.default.yellowBright('Compiled with warnings'));
            warnings.forEach(warning => {
                console.log(chalk_1.default.yellowBright(warning));
            });
        }
        if (errors.length) {
            console.log(chalk_1.default.redBright('Compiled with errors'));
            errors.forEach(error => {
                console.log(chalk_1.default.redBright(error));
            });
            hasError = true;
            (_b = childProcess) === null || _b === void 0 ? void 0 : _b.kill(1);
            process.exit(1);
        }
        hasError = false;
        // [app.js, app.js.map]
        const assets = (_c = assetsByChunkName) === null || _c === void 0 ? void 0 : _c.app;
        if (outputPath && path_1.default.basename(outputPath) === 'client') {
            return;
        }
        chunkPath = path_1.default.join(outputPath, assets[0]);
        buildTime = time;
        if (childProcess) {
            childProcess.kill();
        }
        else {
            createChildProcess();
        }
    });
}
function build() {
    var _a, _b;
    const serverDevConfig = webpack_config_1.createWebpackConfig(true);
    const clientDevConfig = webpack_config_1.createWebpackConfig(false);
    // clear dist directory and copy static files.
    const serverOutput = (_a = serverDevConfig.output) === null || _a === void 0 ? void 0 : _a.path;
    const clientOutput = (_b = clientDevConfig.output) === null || _b === void 0 ? void 0 : _b.path;
    utils_1.deleteDir(serverOutput);
    utils_1.deleteDir(clientOutput);
    const publicDir = path_1.default.join(config.rootDir, 'public');
    utils_1.copyDir(publicDir, serverOutput);
    utils_1.copyDir(publicDir, clientOutput);
    runCompile([serverDevConfig, clientDevConfig]);
}
build();
